/*
    JV Library
    Copyright (C) 2014  Jouko Juhani Virtanen

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
# include <vector>

using namespace std;

# include "Structures.h"
# include "AtomCode.h"
# include "IOUtils.h"
//# include "StringUtils.h"
//# include "LinkedList.h"

void printError(string resstr, string str)
{
	cout <<"Error unable to identify solvent type"<<endl;
	cout <<resstr<<"	"<<str<<endl;
}

int GetSolventType2(string resstr, string str)
{
        bool verbose=false;
        string str1, str2, str2b, str3, str_trim;
        Delete(str, " ");
        str1=str[0];
        str2=str.substr(0,2);
        str3=str.substr(0,3);
        if (str1=="H")
        {

                if (str2=="HA") return HC;
                else if (str2=="HB") return HC;
                else if (str2=="HN") return HN;
                else if (str2=="HT") return HN;

                else if (resstr=="ARG")
                {
                        if (str2=="HG") return HC;
                        else if (str2=="HD") return HC;
                        else if (str2=="HE") return HN;
                        else if (str2=="HH") return HN;
                        else
                        {
				if (verbose) printError(resstr, str);
                                return HC;
                        }
                }

                else if (resstr=="ASN")
                {
                        if (str2=="HD") return HN;
                        else
                        {
				if (verbose) printError(resstr, str);
                                return HC;
                        }
                }

                else if (resstr=="CYS")
                {
                        if (str2=="HG") return HS;
                        else 
                        {
				if (verbose) printError(resstr, str);
                                return HC;
                        }
                }
                else if (resstr=="GLN")
                {
                        if (str2=="HG") return HC;
                        else if (str2=="HE") return HN;
                        else
                        {
				if (verbose) printError(resstr, str);
                                return HC;
                        }
                }

                else if (resstr=="GLU")
                {
                        if (str2=="HG") return HC;
                        else
                        {
				if (verbose) printError(resstr, str);
                                return HC;
                        }
                }

                else if (resstr=="HSP" || resstr=="HSE" || resstr=="HSD")
                {
                        if (str3=="HD2") return HC;
                        else if (str3=="HE2") return HN;
                        else if (str3=="HD1") return HN;
                        else if (str3=="HE1") return HC;
                        else
                        {
				if (verbose) printError(resstr, str);
                                return HC;
                        }
                }

                else if (resstr=="ILE")
                {
                        if (str2=="NH") return HN;
                        else if (str2=="HG") return HC;
                        else if (str2=="HD") return HC;
                        else
                        {
				if (verbose) printError(resstr, str);
                                return HC;
                        }
                }

                else if (resstr=="LEU")
                {
                        if (str2!="NH") return HC;
                        else
                        {
				if (verbose) printError(resstr, str);
                                return HC;
                        }
                }

                else if (resstr=="LYS")
                {
                        if (str2=="HG") return HC;
                        else if (str2=="HD") return HC;
                        else if (str2=="HE") return HC;
                        else if (str2=="HZ") return HN;
                        else
                        {
				if (verbose) printError(resstr, str);
                                cout <<"Error unable to identify solvent type"<<endl;
                                cout <<resstr<<"	"<<str<<endl;
                                return HC;
                        }
                }

                else if (resstr=="MET")
                {
                        if (str2=="HT") return HN;
                        else if (str2=="HA") return HC;
                        else if (str2=="HB") return HC;
                        else if (str2=="HE") return HC;
                        else if (str2=="HG") return HC;
                        //Needs to be fixed.  Deleted CE.
                        else
                        {
				if (verbose) printError(resstr, str);
                                return HC;
                        }
                }

                else if (resstr=="PHE")
                {
                        if (str2=="HN") return HN;
                        else if (str2=="HD") return HC;
                        else if (str2=="HE") return HC;
                        else if (str2=="HZ") return HC;
                        else
                        {
				if (verbose) printError(resstr, str);
                                return HC;
                        }
                }

                else if (resstr=="PRO")
                {
                        if (str2!="HN") return HC;
                        else
                        {
				if (verbose) printError(resstr, str);
                                return HC;
                        }
                }

                else if (resstr=="THR")
                {
                        if (str3=="HG1") return HO;
                        else if (str3=="HG2") return HC;
                        else
                        {
				if (verbose) printError(resstr, str);
                                return HC;
                        }
                }

                else if (resstr=="VAL")
                {
                        if (str2!="HN") return HC;
                        else
                        {
				if (verbose) printError(resstr, str);
                                return HC;
                        }
                }

                else if (resstr=="SER")
                {
                        if (str2=="HG") return HO;
                        else
                        {
				if (verbose) printError(resstr, str);
                                return HC;
                        }
                }

                else if (resstr=="TRP")
                {
                        if (str2=="HA") return HC;
                        else if (str2=="HD") return HC;
                        else if (str3=="HE1") return HN;
                        else if (str3=="HE3") return HC;
                        else if (str2=="HZ") return HC;
                        else if (str2=="HH") return HC;
                        else
                        {
				if (verbose) printError(resstr, str);
                                return HC;
                        }
                }

                else if (resstr=="TYR")
                {
                        if (str2=="HD") return HC;
                        else if (str2=="HE") return HC;
                        else if (str2=="HH") return HO;
                        else
                        {
				if (verbose) printError(resstr, str);
                                return HC;
                        }
                }
                else if (resstr=="HEM")
                {
                        if (str2=="HC") return HC;
                        else if (str2=="HD") return HC;
                        else if (str2=="HM") return HC;
                        else
                        {
				if (verbose) printError(resstr, str);
                                return HC;
                        }
                }
                else if (resstr=="GUA")
                {
                        //Check to make sure that this is correct
                        if (str=="H5'") return HC;
                        else if (str=="H5''") return HC;
                        else if (str=="H3'") return HC;
                        else if (str=="H4'") return HC;
                        else if (str=="H1'") return HC;
                        else if (str=="H8") return HC;
                        else if (str=="H1") return HN;
                        else if (str=="H21") return HN;
                        else if (str=="H22") return HN;
                        else if (str=="H3") return HC;
                        else if (str=="H2''") return HC;
                        else if (str=="H2'") return HO;
                        else
                        {

                                if (verbose) cout <<"Error unable to identify solvent type.  Residue= "<<resstr<<" str= "<<str<<endl;
                                return UNK_ATOM_TYPE;
                        }
                }
                else if (resstr=="URA")
                {
                        if (str=="H5'") return HC;
                        else if (str=="H5''") return HC;
                        else if (str=="H4'") return HC;
                        else if (str=="H1'") return HC;
                        else if (str=="H6") return HC;
                        else if (str=="H5") return HC;
                        else if (str=="H3") return HN;
                        else if (str=="H3'") return HC;
                        else if (str=="H2''") return HC;
                        else if (str=="H2'") return HO;
                        else
                        {

                                if (verbose) cout <<"Error unable to identify solvent type.  Residue= "<<resstr<<" str= "<<str<<endl;
                                return UNK_ATOM_TYPE;
                        }
                }
                else if (resstr=="ADE")
                {
                        if (str=="H5'") return HC;
                        else if (str=="H5''") return HC;
                        else if (str=="H4'") return HC;
                        else if (str=="H1'") return HC;
                        else if (str=="H8") return HC;
                        else if (str=="H61") return HN;
                        else if (str=="H62") return HN;
                        else if (str=="H2") return HC;
                        else if (str=="H3'") return HC;
                        else if (str=="H2''") return HC;
                        else if (str=="H2'") return HO;
                        else
                        {

                                if (verbose) cout <<"Error unable to identify solvent type.  Residue= "<<resstr<<" str= "<<str<<endl;
                                return UNK_ATOM_TYPE;
                        }
                }
                else if (resstr=="CYT")
                {
                        if (str=="H5'") return HC;
                        else if (str=="H5''") return HC;
                        else if (str=="H4'") return HC;
                        else if (str=="H1'") return HC;
                        else if (str=="H6") return HC;
                        else if (str=="H5") return HC;
                        else if (str=="H41") return HN;
                        else if (str=="H42") return HN;
                        else if (str=="H3'") return HC;
                        else if (str=="H2''") return HC;
                        else if (str=="H2'") return HO;
                        else
                        {

                                if (verbose) cout <<"Error unable to identify solvent type.  Residue= "<<resstr<<" str= "<<str<<endl;
                                return UNK_ATOM_TYPE;
                        }
                }
                else
                {
                        if (verbose) cout <<"Error unable to identify atomtype2."
                                <<resstr<<"	"<<str<<endl;
                        return UNK_ATOM_TYPE;
                }
        }

        else if (str1=="C") return C;
        else if (str1=="N") return N;
        else if (str1=="O") return O;
        else if (str1=="S") return S;
        else if (str1=="P") return P;
        else
        {
                if (verbose) cout <<"Error unable to identify solvent type"<<endl;
                if (verbose) cout <<resstr<<"	"<<str<<endl;
                return UNK_ATOM_TYPE;
        }

}

void ElementTypeToElementName(string ElementName[])
{
        ElementName[HC]="HC";
        ElementName[HN]="HN";
        ElementName[HO]="HO";
        ElementName[HS]="HS";
        ElementName[C]="C";
        ElementName[N]="N";
        ElementName[O]="O";
        ElementName[S]="S";
}

void getAtomType2(vector<AtomStruct> &Atoms)
{
        int i, natom=Atoms.size();
        for (i=0;i<natom;i++)
        {
                if (Atoms[i].ResidueName!="TIP3")
                {
                        Atoms[i].AtomType2=GetSolventType2(Atoms[i].ResidueName, Atoms[i].AtomName);
                }
        }
}

