/*
    JV Library
    Copyright (C) 2014  Jouko Juhani Virtanen

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _MonteCarlo_include_
#define _MonteCarlo_include_

struct MoveStruct
{
	int nMoveTypes;
	vector<int> index;
	Vector score, dScore, scoreAccept, temp, size, acceptProb, width;
	Vector moveSizeProb, probability, probSum, time;
};

Real updateMoveSize(MoveStruct &move, int numToUse, int index);
void updateMoveSizes(MoveStruct &move, int nMoveTypes);
void updateMoveProbabilities(MoveStruct &move, int nMoveTypes);
void updateTemperatures(vector<MoveStruct> &moves, Vector &temps, Real &targetSwapRate, Real targetTemperature);
void makeRandomChange(Vector &coefficient, MoveStruct &move);
void makeRandomChange2(Vector &coefficient);
Real calcAcceptProb(Real dScore, Real temp);
Real calcAcceptRate(MoveStruct &move);
Real calcStdDev(MoveStruct &move);
void setTemps(Vector &temps, Real minTemp, Real maxTemp, int nrep);
void initializeCoefficients(Vector &coefficient, Matrix &coefficients, int nrep);
void initializeMoveProbabilities(MoveStruct &move, int nMoveTypes);
void initializeMove(Vector &coefficient, MoveStruct &move);
void initializeMoves(Matrix &coefficients, vector<MoveStruct> &moves);
Real calcSwapProb(Real score1, Real score2, Real temp1, Real temp2);
Real calcAveSwapProb(MoveStruct &move1, MoveStruct &move2, Real temp1, Real temp2);
bool swapTemps(Vector &temps, Vector &score, Matrix &coefficients, Vector &totalSwaps, Vector &swapAttempts, int cycle);
void fixFractionHot(Vector &fractionHot);
void calcFractionHot(Vector &hotHistogram, Vector &coldHistogram, Vector &fractionHot);
void calcFractionCold(Vector &hotHistogram, Vector &coldHistogram, Vector &fractionCold);
void initializeHotAndCold(vector<int> &hotOrCold, Vector &hotHistogram, Vector &coldHistogram, int nrep);
void updateTemperatureUsingHotHistogram(Vector &fractionHot, Vector &temps);
void multipleSwaps(Matrix &coefficients, Vector &scores, Vector &totalSwaps, Vector &swapAttempts, Vector &temps, vector<int> &hotOrCold, Vector &hotHistogram, Vector &coldHistogram, int iterations);
void multipleSwaps(Matrix &coefficients, Vector &scores, Vector &totalSwaps, Vector &swapAttempts, Vector &temps, vector<int> &hotOrCold, Vector &hotHistogram, Vector &coldHistogram);
void calcSwapRates(Vector &totalSwaps, Vector &swapAttempts, Vector &swapRates);
void printSwapRates(Vector &totalSwaps, Vector &swapAttempts);
void printMoveSizes(MoveStruct &move);
void printMoveSizes(vector<MoveStruct> &moves);
void printMoveProbabilities(MoveStruct &move);
void printMoveProbabilities(vector<MoveStruct> &moves);
#endif
